import bluepy.btle as btle

from airthings.constants import (
    SENSOR_ATMOSPHERIC_PRESSURE_KEY,
    SENSOR_CO2_KEY,
    SENSOR_HUMIDITY_KEY,
    SENSOR_RADON_LONG_TERM_AVG_KEY,
    SENSOR_RADON_SHORT_TERM_AVG_KEY,
    SENSOR_TEMPERATURE_KEY,
    SENSOR_VOC_KEY,
    WAVE_GEN_1_KEY,
    WAVE_GEN_1_LABEL,
    WAVE_GEN_1_MODEL_NUMBER,
    WAVE_GEN_1_RAW_DATA_FORMAT,
    WAVE_GEN_1_UUID_DATETIME,
    WAVE_GEN_1_UUID_HUMIDITY,
    WAVE_GEN_1_UUID_RADON_LONG_TERM_AVERAGE,
    WAVE_GEN_1_UUID_RADON_SHORT_TERM_AVERAGE,
    WAVE_GEN_1_UUID_TEMPERATURE,
)
from airthings.models import Device
from airthings.sensors import (
    AtmosphericPressureSensor,
    CO2Sensor,
    HumiditySensor,
    RadonLongTermAverageSensor,
    RadonShortTermAverageSensor,
    TemperatureSensor,
    VOCSensor,
)

from .utils import parse_radon_data


class WaveGen1(Device):
    """
    On 1st Gen Wave, temperature and humidity are updated every time we read the wave.
    Radon measurements are updated once every hour.
    """

    KEY = WAVE_GEN_1_KEY
    MODEL_NUMBER = WAVE_GEN_1_MODEL_NUMBER
    LABEL = WAVE_GEN_1_LABEL
    RAW_DATA_FORMAT = WAVE_GEN_1_RAW_DATA_FORMAT
    UUID_DATETIME = btle.UUID(WAVE_GEN_1_UUID_DATETIME)
    UUID_HUMIDITY = btle.UUID(WAVE_GEN_1_UUID_HUMIDITY)
    UUID_TEMPERATURE = btle.UUID(WAVE_GEN_1_UUID_TEMPERATURE)
    UUID_RADON_STA = btle.UUID(WAVE_GEN_1_UUID_RADON_SHORT_TERM_AVERAGE)
    UUID_RADON_LTA = btle.UUID(WAVE_GEN_1_UUID_RADON_LONG_TERM_AVERAGE)
    SENSOR_CAPABILITIES = {
        SENSOR_HUMIDITY_KEY: True,
        SENSOR_RADON_SHORT_TERM_AVG_KEY: True,
        SENSOR_RADON_LONG_TERM_AVG_KEY: True,
        SENSOR_TEMPERATURE_KEY: True,
        SENSOR_ATMOSPHERIC_PRESSURE_KEY: False,
        SENSOR_CO2_KEY: False,
        SENSOR_VOC_KEY: False,
    }

    def _fetch_raw_data(self):
        periph = btle.Peripheral(self.mac_address)
        datetime_char = periph.getCharacteristics(uuid=self.UUID_DATETIME)[0]
        humidity_char = periph.getCharacteristics(uuid=self.UUID_HUMIDITY)[0]
        temperature_char = periph.getCharacteristics(uuid=self.UUID_TEMPERATURE)[0]
        radon_sta_char = periph.getCharacteristics(uuid=self.UUID_RADON_STA)[0]
        radon_lta_char = periph.getCharacteristics(uuid=self.UUID_RADON_LTA)[0]
        raw_data = datetime_char.read()
        raw_data += humidity_char.read()
        raw_data += temperature_char.read()
        raw_data += radon_sta_char.read()
        raw_data += radon_lta_char.read()
        periph.disconnect()
        return raw_data

    def _parse_data(self, data):
        self._measurements[SENSOR_HUMIDITY_KEY] = HumiditySensor(data[6] / 100.0)
        self._measurements[SENSOR_TEMPERATURE_KEY] = TemperatureSensor(data[7] / 100.0)
        self._measurements[
            SENSOR_RADON_SHORT_TERM_AVG_KEY
        ] = RadonShortTermAverageSensor(parse_radon_data(data[8]))
        self._measurements[SENSOR_RADON_LONG_TERM_AVG_KEY] = RadonLongTermAverageSensor(
            parse_radon_data(data[9])
        )



from .atmospheric_pressure import AtmosphericPressureSensor
from .co2 import CO2Sensor
from .humidity import HumiditySensor
from .radon_long_term_average import RadonLongTermAverageSensor
from .radon_short_term_average import RadonShortTermAverageSensor
from .temperature import TemperatureSensor
from .voc import VOCSensor



import struct

import bluepy.btle as btle

from .constants import (
    DEVICE_IDENTIFIER_LENGTH,
    DEVICE_MODEL_NUMBER_LENGTH,
    SENSOR_ATMOSPHERIC_PRESSURE_KEY,
    SENSOR_CO2_KEY,
    SENSOR_HUMIDITY_KEY,
    SENSOR_RADON_LONG_TERM_AVG_KEY,
    SENSOR_RADON_SHORT_TERM_AVG_KEY,
    SENSOR_TEMPERATURE_KEY,
    SENSOR_VOC_KEY,
)


class Sensor:
    KEY = None
    LABEL = None
    UNIT = None

    def __init__(self, value):
        self._value = value

    def __repr__(self):
        return repr("%s %s" % (self._value, self.unit))

    @property
    def key(self):
        return self.KEY

    @property
    def label(self):
        return self.LABEL

    @property
    def unit(self):
        return self.UNIT

    @property
    def value(self):
        return self._value


class Device:
    KEY = None
    MODEL_NUMBER = None
    LABEL = None
    RAW_DATA_FORMAT = None
    SENSOR_CAPABILITIES = {
        SENSOR_HUMIDITY_KEY: False,
        SENSOR_RADON_SHORT_TERM_AVG_KEY: False,
        SENSOR_RADON_LONG_TERM_AVG_KEY: False,
        SENSOR_TEMPERATURE_KEY: False,
        SENSOR_ATMOSPHERIC_PRESSURE_KEY: False,
        SENSOR_CO2_KEY: False,
        SENSOR_VOC_KEY: False,
    }

    def __init__(self, mac_address, serial_number):
        self._mac_address = mac_address
        self._serial_number = serial_number
        self._identifier = serial_number[-DEVICE_IDENTIFIER_LENGTH:]
        self._measurements = {}
        self._has_measurements = False

    def __repr__(self):
        return repr(
            "<Device mac_address=%s serial_number=%s model_number=%s model=%s>"
            % (self.mac_address, self.serial_number, self.model_number, self.LABEL,)
        )

    def _fetch_raw_data(self):
        periph = btle.Peripheral(self.mac_address)
        characteristics = periph.getCharacteristics(uuid=self.DATA_UUID)
        if len(characteristics) != 1:
            raise ValueError(
                "getCharacteristics did not return exactly 1 characteristic"
            )

        characteristic = characteristics[0]
        raw_data = characteristic.read()
        periph.disconnect()
        return raw_data

    def _parse_raw_data(self, raw_data):
        return struct.unpack(self.RAW_DATA_FORMAT, raw_data)

    def fetch_and_set_measurements(self):
        raw_data = self._fetch_raw_data()
        data = self._parse_raw_data(raw_data)
        # TODO: check sensor version
        self._parse_data(data)
        self._has_measurements = True

    @property
    def mac_address(self):
        return self._mac_address

    @mac_address.setter
    def mac_address(self, mac_address):
        self._mac_address = mac_address

    @property
    def serial_number(self):
        return self._serial_number

    @serial_number.setter
    def serial_number(self, serial_number):
        self._serial_number = serial_number

    @property
    def identifier(self):
        return self._identifier

    @property
    def model_number(self):
        return self.MODEL_NUMBER

    @property
    def sensor_capabilities(self):
        return self.SENSOR_CAPABILITIES

    @property
    def has_measurements(self):
        return self._has_measurements

    @property
    def label(self):
        return self.LABEL

    @property
    def measurements(self):
        return self._measurements

    @property
    def humidity(self):
        return self._measurements.get(SENSOR_HUMIDITY_KEY, None)

    @property
    def radon_short_term_avg(self):
        return self._measurements.get(SENSOR_RADON_SHORT_TERM_AVG_KEY, None)

    @property
    def radon_long_term_avg(self):
        return self._measurements.get(SENSOR_RADON_LONG_TERM_AVG_KEY, None)

    @property
    def temperature(self):
        return self._measurements.get(SENSOR_TEMPERATURE_KEY, None)

    @property
    def atmospheric_pressure(self):
        return self._measurements.get(SENSOR_ATMOSPHERIC_PRESSURE_KEY, None)

    @property
    def co2(self):
        return self._measurements.get(SENSOR_CO2_KEY, None)

    @property
    def voc(self):
        return self._measurements.get(SENSOR_VOC_KEY, None)



import bluepy.btle as btle

from airthings.constants import (
    SENSOR_ATMOSPHERIC_PRESSURE_KEY,
    SENSOR_CO2_KEY,
    SENSOR_HUMIDITY_KEY,
    SENSOR_RADON_LONG_TERM_AVG_KEY,
    SENSOR_RADON_SHORT_TERM_AVG_KEY,
    SENSOR_TEMPERATURE_KEY,
    SENSOR_VOC_KEY,
    WAVE_MINI_GEN_1_KEY,
    WAVE_MINI_GEN_1_LABEL,
    WAVE_MINI_GEN_1_MODEL_NUMBER,
    WAVE_MINI_GEN_1_RAW_DATA_FORMAT,
    WAVE_MINI_GEN_1_UUID_DATA,
)
from airthings.models import Device
from airthings.sensors import (
    AtmosphericPressureSensor,
    HumiditySensor,
    TemperatureSensor,
    VOCSensor,
)

from .utils import parse_radon_data


class WaveMiniGen1(Device):
    """
    Sensor values are updated every 5 minutes.
    """

    KEY = WAVE_MINI_GEN_1_KEY
    MODEL_NUMBER = WAVE_MINI_GEN_1_MODEL_NUMBER
    LABEL = WAVE_MINI_GEN_1_LABEL
    RAW_DATA_FORMAT = WAVE_MINI_GEN_1_RAW_DATA_FORMAT
    DATA_UUID = btle.UUID(WAVE_MINI_GEN_1_UUID_DATA)
    SENSOR_CAPABILITIES = {
        SENSOR_HUMIDITY_KEY: True,
        SENSOR_RADON_SHORT_TERM_AVG_KEY: False,
        SENSOR_RADON_LONG_TERM_AVG_KEY: False,
        SENSOR_TEMPERATURE_KEY: True,
        SENSOR_ATMOSPHERIC_PRESSURE_KEY: False,
        SENSOR_CO2_KEY: False,
        SENSOR_VOC_KEY: True,
    }

    def _parse_data(self, data):
        self._measurements[SENSOR_TEMPERATURE_KEY] = TemperatureSensor(
            round(data[1] / 100.0 - 273.15, 2)
        )
        self._measurements[SENSOR_HUMIDITY_KEY] = HumiditySensor(data[3] / 100.0)
        self._measurements[SENSOR_VOC_KEY] = VOCSensor(data[4])



import struct
import sys
import time
from pprint import pprint

import bluepy.btle as btle

from .utils import determine_device_model

# General
SCAN_TIMEOUT = 3  # Seconds
SCAN_RETRIES = 3  # Times
CONNECTION_RETRIES = 3  # Times


def discover_devices(
    mac_addresses=None,
    identifiers=None,
    scan_timeout=SCAN_TIMEOUT,
    scan_retries=SCAN_RETRIES,
):
    """
    Discover Airthings devices either automatically, by MAC addresses or by identifiers
    """
    airthings_devices = []
    current_retries = 0
    while True:
        try:
            scanner = btle.Scanner()
            devices = scanner.scan(scan_timeout)
            for dev in devices:
                device = determine_device_model(dev)
                if device:
                    if mac_addresses and device.mac_address not in mac_addresses:
                        # MAC addresses are set, and the device MAC address does not
                        # match any in the list.
                        continue
                    elif identifiers and device.identifier not in identifiers:
                        # Identifiers are set, and the device identifier does not match
                        # any in the list.
                        continue

                    # Device is an Airthings device
                    airthings_devices.append(device)
            break
        except Exception as _:  # noqa: F841
            # TODO: better error handling
            if current_retries == scan_retries:
                break
            current_retries += 1

    return airthings_devices


def find_devices_by_mac_addresses(
    mac_addresses, scan_timeout=SCAN_TIMEOUT, scan_retries=SCAN_RETRIES
):
    """
    Find Airthings devices by using a list of MAC addresses.
    """
    return discover_devices(
        mac_addresses=mac_addresses,
        scan_timeout=SCAN_TIMEOUT,
        scan_retries=SCAN_RETRIES,
    )


def find_device_by_mac_address(
    mac_address, scan_timeout=SCAN_TIMEOUT, scan_retries=SCAN_RETRIES
):
    """
    Find a single Airthings device by searching for a MAC address.
    """
    devices = find_devices_by_mac_addresses(
        mac_addresses=[mac_address],
        scan_timeout=scan_timeout,
        scan_retries=scan_retries,
    )
    return devices[0] if devices else None


def find_devices_by_identifiers(
    identifiers, scan_timeout=SCAN_TIMEOUT, scan_retries=SCAN_RETRIES
):
    """
    Find Airthings devices by using a list of identifiers (the last 6 digits of a serial number).
    """
    return discover_devices(
        identifiers=identifiers, scan_timeout=SCAN_TIMEOUT, scan_retries=SCAN_RETRIES
    )


def find_device_by_identifier(identifier, timeout=SCAN_TIMEOUT):
    """
    Find a single Airthings device by using an identifier (the last 6 digits of aserial number).
    """
    devices = find_devices_by_identifiers([identifier])
    return devices[0] if devices else None


def fetch_measurements_from_devices(devices, connection_retries=CONNECTION_RETRIES):
    """
    Fetch measurements from a list of Airthings devices.
    """
    current_retries = 0
    for device in devices:
        while True:
            try:
                device.fetch_and_set_measurements()
                break
            except btle.BTLEDisconnectError as _:  # noqa: F841
                # TODO: better error handling
                if current_retries == connection_retries:
                    break
                current_retries += 1

    return devices


def fetch_measurements(
    mac_addresses=None,
    identifiers=None,
    scan_timeout=SCAN_TIMEOUT,
    scan_retries=SCAN_RETRIES,
    connection_retries=CONNECTION_RETRIES,
):
    """
    Fetch measurements from Airthings devices either automatically, by MAC addresses or by identifiers
    """

    airthings_devices = discover_devices(
        mac_addresses=mac_addresses,
        identifiers=identifiers,
        scan_timeout=scan_timeout,
        scan_retries=scan_retries,
    )

    return fetch_measurements_from_devices(
        devices=airthings_devices, connection_retries=connection_retries
    )


def fetch_measurements_from_identifiers(
    identifiers,
    scan_timeout=SCAN_TIMEOUT,
    scan_retries=SCAN_RETRIES,
    connection_retries=CONNECTION_RETRIES,
):
    """
    Fetch measurements from a list of Airthings device identifiers.
    """

    return fetch_measurements(
        identifiers=identifiers,
        scan_timeout=scan_timeout,
        scan_retries=scan_retries,
        connection_retries=connection_retries,
    )


def fetch_measurements_from_identifier(
    identifier,
    scan_timeout=SCAN_TIMEOUT,
    scan_retries=SCAN_RETRIES,
    connection_retries=CONNECTION_RETRIES,
):
    """
    Fetch measurements from a specific Airthings device identifier.
    """
    devices = fetch_measurements_from_identifiers(
        identifiers=[identifier],
        scan_timeout=scan_timeout,
        scan_retries=scan_retries,
        connection_retries=connection_retries,
    )
    return devices[0] if devices else None


def fetch_measurements_from_mac_addresses(
    mac_addresses,
    scan_timeout=SCAN_TIMEOUT,
    scan_retries=SCAN_RETRIES,
    connection_retries=CONNECTION_RETRIES,
):
    """
    Fetch measurements from a list of Airthings device MAC addresses.
    """

    return fetch_measurements(
        mac_addresses=mac_addresses,
        scan_timeout=scan_timeout,
        scan_retries=scan_retries,
        connection_retries=connection_retries,
    )


def fetch_measurements_from_mac_address(
    mac_address,
    scan_timeout=SCAN_TIMEOUT,
    scan_retries=SCAN_RETRIES,
    connection_retries=CONNECTION_RETRIES,
):
    """
    Fetch measurements from a specific Airthings device MAC address.
    """
    devices = fetch_measurements(
        mac_addresses=[mac_address],
        scan_timeout=scan_timeout,
        scan_retries=scan_retries,
        connection_retries=connection_retries,
    )
    return devices[0] if devices else None



from airthings.constants import (
    SENSOR_ATMOSPHERIC_PRESSURE_KEY,
    SENSOR_ATMOSPHERIC_PRESSURE_LABEL,
    SENSOR_ATMOSPHERIC_PRESSURE_UNIT,
)
from airthings.models import Sensor


class AtmosphericPressureSensor(Sensor):
    KEY = SENSOR_ATMOSPHERIC_PRESSURE_KEY
    LABEL = SENSOR_ATMOSPHERIC_PRESSURE_LABEL
    UNIT = SENSOR_ATMOSPHERIC_PRESSURE_UNIT

    def __init__(self, value):
        super(AtmosphericPressureSensor, self).__init__(value)



from airthings.constants import (
    SENSOR_TEMPERATURE_KEY,
    SENSOR_TEMPERATURE_LABEL,
    SENSOR_TEMPERATURE_UNIT,
)
from airthings.models import Sensor


class TemperatureSensor(Sensor):
    KEY = SENSOR_TEMPERATURE_KEY
    LABEL = SENSOR_TEMPERATURE_LABEL
    UNIT = SENSOR_TEMPERATURE_UNIT

    def __init__(self, value):
        super(TemperatureSensor, self).__init__(value)



from airthings.constants import (
    SENSOR_HUMIDITY_KEY,
    SENSOR_HUMIDITY_LABEL,
    SENSOR_HUMIDITY_UNIT,
)
from airthings.models import Sensor


class HumiditySensor(Sensor):
    KEY = SENSOR_HUMIDITY_KEY
    LABEL = SENSOR_HUMIDITY_LABEL
    UNIT = SENSOR_HUMIDITY_UNIT

    def __init__(self, value):
        super(HumiditySensor, self).__init__(value)



import bluepy.btle as btle

from airthings.constants import (
    SENSOR_ATMOSPHERIC_PRESSURE_KEY,
    SENSOR_CO2_KEY,
    SENSOR_HUMIDITY_KEY,
    SENSOR_RADON_LONG_TERM_AVG_KEY,
    SENSOR_RADON_SHORT_TERM_AVG_KEY,
    SENSOR_TEMPERATURE_KEY,
    SENSOR_VOC_KEY,
    WAVE_GEN_2_KEY,
    WAVE_GEN_2_LABEL,
    WAVE_GEN_2_MODEL_NUMBER,
    WAVE_GEN_2_RAW_DATA_FORMAT,
    WAVE_GEN_2_UUID_DATA,
)
from airthings.models import Device
from airthings.sensors import (
    AtmosphericPressureSensor,
    CO2Sensor,
    HumiditySensor,
    RadonLongTermAverageSensor,
    RadonShortTermAverageSensor,
    TemperatureSensor,
    VOCSensor,
)

from .utils import parse_radon_data


class WaveGen2(Device):
    """
    On 2nd Gen Wave, temperature and humidity are updated every 5 minutes.
    Radon measurements are updated once every hour.
    """

    KEY = WAVE_GEN_2_KEY
    MODEL_NUMBER = WAVE_GEN_2_MODEL_NUMBER
    LABEL = WAVE_GEN_2_LABEL
    RAW_DATA_FORMAT = WAVE_GEN_2_RAW_DATA_FORMAT
    DATA_UUID = btle.UUID(WAVE_GEN_2_UUID_DATA)
    SENSOR_CAPABILITIES = {
        SENSOR_HUMIDITY_KEY: True,
        SENSOR_RADON_SHORT_TERM_AVG_KEY: True,
        SENSOR_RADON_LONG_TERM_AVG_KEY: True,
        SENSOR_TEMPERATURE_KEY: True,
        SENSOR_ATMOSPHERIC_PRESSURE_KEY: False,
        SENSOR_CO2_KEY: False,
        SENSOR_VOC_KEY: False,
    }

    def _parse_data(self, data):
        self._measurements[SENSOR_HUMIDITY_KEY] = HumiditySensor(data[1] / 2.0)
        self._measurements[
            SENSOR_RADON_SHORT_TERM_AVG_KEY
        ] = RadonShortTermAverageSensor(parse_radon_data(data[4]))
        self._measurements[SENSOR_RADON_LONG_TERM_AVG_KEY] = RadonLongTermAverageSensor(
            parse_radon_data(data[5])
        )
        self._measurements[SENSOR_TEMPERATURE_KEY] = TemperatureSensor(data[6] / 100.0)



from airthings.constants import SENSOR_VOC_KEY, SENSOR_VOC_LABEL, SENSOR_VOC_UNIT
from airthings.models import Sensor


class VOCSensor(Sensor):
    KEY = SENSOR_VOC_KEY
    LABEL = SENSOR_VOC_LABEL
    UNIT = SENSOR_VOC_UNIT

    def __init__(self, value):
        super(VOCSensor, self).__init__(value)



from airthings.constants import SENSOR_CO2_KEY, SENSOR_CO2_LABEL, SENSOR_CO2_UNIT
from airthings.models import Sensor


class CO2Sensor(Sensor):
    KEY = SENSOR_CO2_KEY
    LABEL = SENSOR_CO2_LABEL
    UNIT = SENSOR_CO2_UNIT

    def __init__(self, value):
        super(CO2Sensor, self).__init__(value)



from airthings.constants import (
    SENSOR_RADON_SHORT_TERM_AVG_KEY,
    SENSOR_RADON_SHORT_TERM_AVG_LABEL,
    SENSOR_RADON_SHORT_TERM_AVG_UNIT,
)
from airthings.models import Sensor


class RadonShortTermAverageSensor(Sensor):
    KEY = SENSOR_RADON_SHORT_TERM_AVG_KEY
    LABEL = SENSOR_RADON_SHORT_TERM_AVG_LABEL
    UNIT = SENSOR_RADON_SHORT_TERM_AVG_UNIT

    def __init__(self, value):
        super(RadonShortTermAverageSensor, self).__init__(value)



import setuptools

try:
    from pypandoc import convert

    read_md = lambda f: convert(f, "rst")
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, "r").read()

setuptools.setup(
    name="airthings",
    version="1.0.2",
    author="Marius Kotlarz",
    author_email="marius@kotlarz.no",
    description="Fetch sensor measurements from Airthings devices",
    long_description=read_md("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/kotlarz/airthings",
    packages=setuptools.find_packages(),
    install_requires=["bluepy==1.3.0"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Home Automation",
        "Topic :: Utilities",
    ],
)



from .wave_gen_1 import WaveGen1
from .wave_gen_2 import WaveGen2
from .wave_mini_gen_1 import WaveMiniGen1
from .wave_plus_gen_1 import WavePlusGen1

DEVICE_MODELS = [
    WaveGen1,
    WaveGen2,
    WaveMiniGen1,
    WavePlusGen1,
]



# General
DEVICE_MODEL_NUMBER_LENGTH = 4
DEVICE_IDENTIFIER_LENGTH = 6

# Sensors
## Humidity
SENSOR_HUMIDITY_KEY = "humidity"
SENSOR_HUMIDITY_LABEL = "Humidity"
SENSOR_HUMIDITY_UNIT = "%rH"
## Radon short term avg
SENSOR_RADON_SHORT_TERM_AVG_KEY = "radon_short_term_avg"
SENSOR_RADON_SHORT_TERM_AVG_LABEL = "Radon Short Term Average"
SENSOR_RADON_SHORT_TERM_AVG_UNIT = "Bq/m3"
## Radon long term avg
SENSOR_RADON_LONG_TERM_AVG_KEY = "radon_long_term_avg"
SENSOR_RADON_LONG_TERM_AVG_LABEL = "Radon Long Term Average"
SENSOR_RADON_LONG_TERM_AVG_UNIT = "Bq/m3"
## Temperature
SENSOR_TEMPERATURE_KEY = "temperature"
SENSOR_TEMPERATURE_LABEL = "Temperature"
SENSOR_TEMPERATURE_UNIT = "°C"
## Atmospheric pressure
SENSOR_ATMOSPHERIC_PRESSURE_KEY = "atmospheric_pressure"
SENSOR_ATMOSPHERIC_PRESSURE_LABEL = "Atmospheric pressure"
SENSOR_ATMOSPHERIC_PRESSURE_UNIT = "hPa"
## CO2
SENSOR_CO2_KEY = "co2"
SENSOR_CO2_LABEL = "CO2"
SENSOR_CO2_UNIT = "ppm"
## VOC
SENSOR_VOC_KEY = "voc"
SENSOR_VOC_LABEL = "VOC"
SENSOR_VOC_UNIT = "ppb"

# Devices

## Wave Gen 1
WAVE_GEN_1_KEY = "wave_gen_1"
WAVE_GEN_1_LABEL = "Wave Gen 1"
WAVE_GEN_1_MODEL_NUMBER = "2900"
WAVE_GEN_1_RAW_DATA_FORMAT = "<H5B4H"
WAVE_GEN_1_UUID_DATETIME = 0x2A08
WAVE_GEN_1_UUID_HUMIDITY = 0x2A6F
WAVE_GEN_1_UUID_TEMPERATURE = 0x2A6E
WAVE_GEN_1_UUID_RADON_SHORT_TERM_AVERAGE = "b42e01aa-ade7-11e4-89d3-123b93f75cba"
WAVE_GEN_1_UUID_RADON_LONG_TERM_AVERAGE = "b42e0a4c-ade7-11e4-89d3-123b93f75cba"

## Wave Mini Gen 1
WAVE_MINI_GEN_1_KEY = "wave_mini_gen_1"
WAVE_MINI_GEN_1_LABEL = "Wave Mini Gen 1"
WAVE_MINI_GEN_1_MODEL_NUMBER = "2920"
WAVE_MINI_GEN_1_RAW_DATA_FORMAT = "<HHHHHHLL"
WAVE_MINI_GEN_1_UUID_DATA = "b42e3b98-ade7-11e4-89d3-123b93f75cba"

## Wave Plus Gen 1
WAVE_PLUS_GEN_1_KEY = "wave_plus_gen_1"
WAVE_PLUS_GEN_1_LABEL = "Wave Plus Gen 1"
WAVE_PLUS_GEN_1_MODEL_NUMBER = "2930"
WAVE_PLUS_GEN_1_RAW_DATA_FORMAT = "BBBBHHHHHHHH"
WAVE_PLUS_GEN_1_UUID_DATA = "b42e2a68-ade7-11e4-89d3-123b93f75cba"

## Wave Gen 2
WAVE_GEN_2_KEY = "wave_gen_2"
WAVE_GEN_2_LABEL = "Wave Gen 2"
WAVE_GEN_2_MODEL_NUMBER = "2950"
WAVE_GEN_2_RAW_DATA_FORMAT = "<4B8H"
WAVE_GEN_2_UUID_DATA = "b42e4dcc-ade7-11e4-89d3-123b93f75cba"



import struct

import bluepy.btle as btle

from airthings.devices import DEVICE_MODELS


def determine_device_model(dev):
    mac_address = dev.addr
    manufacturer_data = dev.getValue(btle.ScanEntry.MANUFACTURER)
    for model in DEVICE_MODELS:
        serial_number = parse_manufacturer_data(manufacturer_data)
        if serial_number is None:
            # Not a Airthings device
            continue
        serial_number = str(serial_number)
        model_number = serial_number[:4]
        if model_number == model.MODEL_NUMBER:
            return model(mac_address, serial_number)
    else:
        return None


def parse_manufacturer_data(manufacturer_data):
    try:
        (idx, serial_number, _) = struct.unpack("<HLH", manufacturer_data)
    except Exception:
        return None
    else:
        if idx == 0x0334:
            return serial_number
